"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _app = require("@octokit/app");

var _rest = require("@octokit/rest");

var _request = require("@octokit/request");

var _config = _interopRequireDefault(require("../config"));

var _ErrorHandler = _interopRequireDefault(require("./ErrorHandler"));

var _GitService = _interopRequireDefault(require("./GitService"));

var _Review = _interopRequireDefault(require("./models/Review"));

var _User = _interopRequireDefault(require("./models/User"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const normalizeResponse = ({
  data
}) => data;

class GitHub extends _GitService.default {
  constructor(options = {}) {
    super(options.username, options.repository, options.branch);
    return (async () => {
      const isAppAuth = _config.default.get('githubAppID') && _config.default.get('githubPrivateKey');

      const isLegacyAuth = _config.default.get('githubToken');

      let authToken;

      if (options.oauthToken) {
        authToken = options.oauthToken;
      } else if (isAppAuth) {
        authToken = await GitHub._authenticate(options.username, options.repository);
      } else if (isLegacyAuth) {
        authToken = _config.default.get('githubToken');
      } else {
        throw new Error('Require an `oauthToken` or `token` option');
      }

      this.api = new _rest.Octokit({
        auth: `token ${authToken}`,
        userAgent: 'Staticman',
        baseUrl: _config.default.get('githubBaseUrl'),
        request: {
          timeout: 5000
        }
      });
      return this;
    })();
  }

  static async _authenticate(username, repository) {
    const app = new _app.App({
      id: _config.default.get('githubAppID'),
      privateKey: _config.default.get('githubPrivateKey'),
      baseUrl: _config.default.get('githubBaseUrl')
    });
    const jwt = app.getSignedJsonWebToken();
    const {
      data
    } = await (0, _request.request)('GET /repos/:owner/:repo/installation', {
      owner: username,
      repo: repository,
      headers: {
        authorization: `Bearer ${jwt}`,
        accept: 'application/vnd.github.machine-man-preview+json'
      }
    });
    const installationId = data.id;
    const token = await app.getInstallationAccessToken({
      installationId
    });
    return token;
  }

  _pullFile(filePath, branch) {
    return this.api.repos.getContents({
      owner: this.username,
      repo: this.repository,
      path: filePath,
      ref: branch
    }).then(normalizeResponse).catch(err => Promise.reject((0, _ErrorHandler.default)('GITHUB_READING_FILE', {
      err
    })));
  }

  _commitFile(filePath, content, commitMessage, branch) {
    return this.api.repos.createOrUpdateFile({
      owner: this.username,
      repo: this.repository,
      path: filePath,
      message: commitMessage,
      content,
      branch
    }).then(normalizeResponse);
  }

  writeFile(filePath, data, targetBranch, commitTitle) {
    return super.writeFile(filePath, data, targetBranch, commitTitle).catch(err => {
      try {
        const message = err === null || err === void 0 ? void 0 : err.message;

        if (message) {
          const parsedError = JSON.parse(message);

          if (parsedError !== null && parsedError !== void 0 && parsedError.message.includes('"sha" wasn\'t supplied')) {
            return Promise.reject((0, _ErrorHandler.default)('GITHUB_FILE_ALREADY_EXISTS', {
              err
            }));
          }
        }
      } catch (errorParsingError) {
        console.log(errorParsingError);
      }

      return Promise.reject((0, _ErrorHandler.default)('GITHUB_WRITING_FILE'));
    });
  }

  getBranchHeadCommit(branch) {
    return this.api.repos.getBranch({
      owner: this.username,
      repo: this.repository,
      branch
    }).then(res => res.data.commit.sha);
  }

  createBranch(branch, sha) {
    return this.api.git.createRef({
      owner: this.username,
      repo: this.repository,
      ref: `refs/heads/${branch}`,
      sha
    }).then(normalizeResponse);
  }

  deleteBranch(branch) {
    return this.api.git.deleteRef({
      owner: this.username,
      repo: this.repository,
      ref: `heads/${branch}`
    });
  }

  createReview(reviewTitle, branch, reviewBody) {
    return this.api.pulls.create({
      owner: this.username,
      repo: this.repository,
      title: reviewTitle,
      head: branch,
      base: this.branch,
      body: reviewBody
    }).then(normalizeResponse);
  }

  getReview(reviewId) {
    return this.api.pulls.get({
      owner: this.username,
      repo: this.repository,
      pull_number: reviewId
    }).then(normalizeResponse).then(({
      base,
      body,
      head,
      merged,
      state,
      title
    }) => new _Review.default(title, body, merged && state === 'closed' ? 'merged' : state, head.ref, base.ref));
  }

  async readFile(filePath, getFullResponse) {
    try {
      return await super.readFile(filePath, getFullResponse);
    } catch (err) {
      throw (0, _ErrorHandler.default)('GITHUB_READING_FILE', {
        err
      });
    }
  }

  writeFileAndSendReview(filePath, data, branch, commitTitle, reviewBody) {
    return super.writeFileAndSendReview(filePath, data, branch, commitTitle, reviewBody).catch(err => Promise.reject((0, _ErrorHandler.default)('GITHUB_CREATING_PR', {
      err
    })));
  }

  getCurrentUser() {
    return this.api.users.getAuthenticated({}).then(normalizeResponse).then(({
      login,
      email,
      avatar_url: avatarUrl,
      name,
      bio,
      company,
      blog
    }) => new _User.default('github', login, email, name, avatarUrl, bio, blog, company)).catch(err => Promise.reject((0, _ErrorHandler.default)('GITHUB_GET_USER', {
      err
    })));
  }

}

exports.default = GitHub;